package org.cytoscape.paperwing.internal.cytoscape.view;

import java.util.HashMap;

import org.cytoscape.model.CyNode;
import org.cytoscape.model.SUIDFactory;
import org.cytoscape.view.model.View;
import org.cytoscape.view.model.VisualProperty;

public class WindNodeView extends VisualPropertyKeeper<CyNode> {

	private CyNode node;
	private Long suid;
	private DefaultValueVault defaultValueVault;
	
	public WindNodeView(DefaultValueVault defaultValueVault, 
			CyNode node) {
		super(defaultValueVault.getVisualLexicon());
		
		this.node = node;
		this.suid = SUIDFactory.getNextSUID();
		this.defaultValueVault = defaultValueVault;
	}
	
	@Override
	public Long getSUID() {
		return suid;
	}

	@Override
	public CyNode getModel() {
		return node;
	}

	@Override
	public <T> T getVisualProperty(VisualProperty<T> visualProperty) {
		T value = super.getVisualProperty(visualProperty);
		
		if (value != null) {
			// If we were given an explicit value, return it
			return value;
		} else {
			// Otherwise, return the default value
			value = defaultValueVault.getDefaultValue(visualProperty);
			
			if (value != null) {
				return value;
			} else {
				return visualProperty.getDefault();
			}
		}
	}
	
	@Override
	protected <T, V extends T> V getDefaultValue(VisualProperty<T> vp) {
		return (V) this.defaultValueVault.getDefaultValue(vp);
	}
}
